///////////////////////////////////////////////////////////////////////
// MathGUI = a WYSIWYG equation editor + a powerful math engine      //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   GUIUtils.h
//! \brief  Defines some useful functions and defines.
//!
//! Main groups of functions defined here are: error-handler functions,
//! global windows access (status bar...), global configuration file
//! access.
///////////////////////////////////////////////////////////////////////



#ifndef MATHGUI_GLOBALS_H
#define MATHGUI_GLOBALS_H


// for borland compiler
#ifndef WX_PRECOMP
#include <wx/wx.h>	// if precompilation is not supported, include wx.h
#endif

#include "mg/Setup.h"




// -------------------
// MACROS
// -------------------

//! Deletes the given pointer and reset it. This macro is wxT('safe') because
//! it checks the argument before deleting it: if it's already NULL,
//! nothing is done.
//! \param x The pointer to the object which must be deleted.
#define mgSAFE_DELETE(x)					{ if (x) delete x; x = NULL; }

//! Like mcSAFE_DELETE() but uses the wxT("delete [] x") istruction.
#define mgSAFE_DELETE_ARRAY(x)				{ if (x) delete [] x; x = NULL; }

#define mgSTILL_TO_IMPLEMENT()											\
	wxMessageBox(wxT("This feature still doesn't work... sorry."),		\
					wxT("Ops"), wxICON_INFORMATION | wxOK);




// -------------------
// DEFINES
// -------------------

// export & import dlgs
#define mgEXPDLG_BASEID					11000
#define mgIMPDLG_BASEID					12000

// palettes
#define mgGREEKDLG_BASEID				13000
#define mgSYMBOLDLG_BASEID				14000
#define mgSPECIALSYMDLG_BASEID			15000
#define mgSOLVEDLG_BASEID				16000

// miscellaneous
#define mgOPTIONSDLG_BASEID				17000
#define mgMATHMENU_BASEID				18000

//! Used to create some new IDs at run-time for the context menu; this
//! value is used as the first ID of a sequence; in this way we're sure
//! our IDs do not create conflicts with wxWidgets IDs.
#define mgMATHBOX_BASEID				19000

//! Applications using MathGUI, to avoid conflicts with MathGUI ids,
//! should use the following, instead of wxID_HIGHEST...
#define mgMATHGUI_HIGHEST_ID			20000


// events ids
#define mgMATHBOX_EVENTS_BASEID				wxEVT_USER_FIRST+100
#define mgOPTIONSDLG_EVENTS_BASEID			wxEVT_USER_FIRST+200
#define mgMATHGUI_GENERAL_EVENTS_BASEID		wxEVT_USER_FIRST+300

#define mgMATHGUI_HIGHEST_EVENTS_ID			wxEVT_USER_FIRST+1000


//! The time each message will remain printed on the status bar (in ms).
#define mgMATHGUI_GLOBAL_MSGWAIT		1000

//! The ID of the global timer used by error.
#define mgMATHGUI_GLOBAL_ERRORTMRID	256

//@}




// ---------------------
// MISCELLANEOUS
// ---------------------

//! Opens the default browser with the given url.
//! Thanks to Julian Smart who put this code in the developer
//! mailing list on the web.
bool wxViewHTMLFile(const wxString& url);



enum MathCoreConfigPath {
	MATHCORE_BASE_CONFIGPATH = -1,
	MATHCORE_APPEARANCE_CONFIGPATH,
	MATHCORE_KEYBINDINGS_CONFIGPATH
};


//! Returns the path which MathCore functions should use to store all the
//! customizable settings in the wxConfig object supplied to the
//! wxCustomizeSystem::LoadCustom & SaveCustom function.
//! This function is not implemented in the MathCore library: it must be
//! implemented by the client application.
//! This function also works as an utility functions because it should check
//! the two given arguments and, if not NULL, they should concatenate them
//! with the constant string (it must NOT change during program execution)
//! representing the group where MathCore will store all its subgroups &
//! entries.
//! The constant string MUST always end with a wxT('/'). It should also begin with
//! a wxT('/'); that is, it should be an absolute path...
wxString GetConfigPath(MathCoreConfigPath id = MATHCORE_BASE_CONFIGPATH, 
					   const wxString &toadd = wxEmptyString);




// --------------------
// MATHGUI LOG
// --------------------

#if defined(mgENABLE_LOG) && \
	!(defined(__GNUG__) && defined(__WXMSW__))	// actually there is an inexplicable bug when using MinGW and mcENABLE_LOG is defined...

	enum {
		mgLOG_Debug = 2048,		// MathGUI log levels start here
		mgLOG_Event,
		mgLOG_BoxEvent,
		mgLOG_User = 200
	};


	#define DECLARE_MATHGUI_LOG_FUNCTION(level)                         \
	extern void WXDLLEXPORT mgVLog##level(const wxChar *szFormat,       \
										  va_list argptr);              \
	extern void WXDLLEXPORT mgLog##level(const wxChar *szFormat,        \
										 ...) ATTRIBUTE_PRINTF_1


	DECLARE_MATHGUI_LOG_FUNCTION(Debug);
	DECLARE_MATHGUI_LOG_FUNCTION(Event);
	DECLARE_MATHGUI_LOG_FUNCTION(BoxEvent);

#else

	#ifdef __GNUG__
	
		// GCC accepts variadic macros and this avoids us a *lot* of
		// warnings about "statements with no effects" that it ouputs
		// if mcLOG is defined to nothing...
		#define mgLogDebug(...)							{;}
		#define mgLogEvent(...)							{;}
		#define mgLogBoxEvent(...)						{;}
	#else

		// If all compilers supported the variadic macros (macros with a variable
		// number of arguments, declared using ... and __VA_ARGS__), then it would
		// be simpler to write these macros !!
		#define mgLogDebug						/* expand to nothing */
		#define mgLogEvent						/* expand to nothing */
		#define mgLogBoxEvent					/* expand to nothing */
	#endif

#endif		// mgENABLE_LOG

#endif		// MATHGUI_GLOBALS_H


