///////////////////////////////////////////////////////////////////////
// MathGUI = a WYSIWYG equation editor + a powerful math engine      //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   GreekDlg.h
//! \brief  Defines the mgGreekPanel class.
//!
//! See GreekDlg.cpp for more info.
///////////////////////////////////////////////////////////////////////


#ifndef GREEKDLG_H
#define GREEKDLG_H

// required includes
#include "mg/MathWnd.h"
#include "mg/MathBox.h"
#include "mg/PaletteDlg.h"
#ifdef mgUSE_PALETTEFRM
	#include <wx/palettefrm.h>
#else
	#include <wx/minifram.h>	
#endif


// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "GreekDlg.h"
#endif



//! The number of small and capital letters in the greek alphabet 
//! which are shown in this dialog window.
#define	GREEKDLG_ALPHABET_LENGHT		24//(0x03C9 - 0x03B1)

//! The number of the columns of buttons of the dialog: it should
//! be a multiple or a divisor of GREEKDLG_ALPHABET_LENGHT.
#define GREEKDLG_GRID_COLUMNS			12
#define GREEKDLG_GRID_ROWS				(GREEKDLG_ALPHABET_LENGHT/GREEKDLG_GRID_COLUMNS)

//! The point size of the font used to draw the greek symbols on
//! the buttons (untied from the point size of the font used to
//! draw the symbol in the mathboxes).
#define GREEKDLG_FONT_POINTSIZE			13

//! The minimum size of the buttons of the panel.
#define GREEKDLG_BUTTON_MINWIDTH		20
#define GREEKDLG_BUTTON_MINHEIGHT		24


//! The buttons base ID (there are GREEKDLG_ALPHABET_LENGHT*2 buttons).
#define IDC_GREEKDLG_SMALLBUTTONS		mgGREEKDLG_BASEID
#define IDC_GREEKDLG_CAPITALBUTTONS		mgGREEKDLG_BASEID+GREEKDLG_ALPHABET_LENGHT




//! A panel containing the greek symbols in small case.
class mgSmallGreekPanel : public mgButtonListPanel
{
	friend class mgCapitalGreekPanel;

	//! The greek alphabet in small and in capital letters.
	static wxString strSmallAlphabet;

	//! The names of the greek letters of the alphabet: alfa, beta, gamma...
	//! They are used for the tooltips.
	static wxString strLetterNames[GREEKDLG_ALPHABET_LENGHT];

protected:

	//! The event handler for the mgGreekPanel buttons.
	void OnButtons( wxCommandEvent &event );

	virtual wxString GetButtonLabel(int i) {
		return strSmallAlphabet.GetChar(i);
	}

	virtual wxString GetButtonTooltip(int i) {
		return wxT("Small ") + strLetterNames[i];
	}

public:

	//! Constructs this dialog. This function works as the standard
	//! wxWindow constructors, with just two new parameters:
	//! \param mathwnd The math window whose selected mathbox receives
	//!                the commands.
	//! \param mathbox The math box which receives the 
	//!                'create greek symbol' commands.
	mgSmallGreekPanel(
		wxWindow *parent,
		wxWindow *client,
		mgMathWnd *mathwnd,
		mgMathBox *mathbox,
		wxWindowID id = -1, 
		const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize,
		long style = wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);

	virtual ~mgSmallGreekPanel() {}
		
private:
	DECLARE_EVENT_TABLE()
};



//! A panel containing the greek symbols in capital case.
class mgCapitalGreekPanel : public mgButtonListPanel
{
	//! The greek alphabet in small and in capital letters.
	static wxString strCapitalAlphabet;

protected:

	//! The event handler for the mgGreekPanel buttons.
	void OnButtons( wxCommandEvent &event );

	virtual wxString GetButtonLabel(int i) {
		return strCapitalAlphabet.GetChar(i);
	}

	virtual wxString GetButtonTooltip(int i) {
		return wxT("Capital ") + mgSmallGreekPanel::strLetterNames[i];
	}

public:

	//! Constructs this dialog. This function works as the standard
	//! wxWindow constructors, with just two new parameters:
	//! \param mathwnd The math window whose selected mathbox receives
	//!                the commands.
	//! \param mathbox The math box which receives the 
	//!                'create greek symbol' commands.
	mgCapitalGreekPanel(
		wxWindow *parent,
		wxWindow *client,
		mgMathWnd *mathwnd,
		mgMathBox *mathbox,
		wxWindowID id = -1, 
		const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize,
		long style = wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);

	virtual ~mgCapitalGreekPanel() {}
		
private:
	DECLARE_EVENT_TABLE()
};



//! A wxPaletteFrame-derived class which.
class mgGreekPalette : public wxPaletteFrame
{
protected:

	//! Build a unique panel grouping the mgSmallGreekPanel and
	//! the mgCapitalGreekPanel panels.
	void Build(wxWindow *client, mgMathWnd *mathwnd, mgMathBox *mathbox);

public:

	mgGreekPalette(
		wxMainFrame *parent,
		wxWindow *client,
		mgMathWnd *mathwnd,
		mgMathBox *mathbox,
		wxWindowID id = -1,
		const wxString &title = wxT("Greek alphabet"),
		const wxPoint &pos = wxDefaultPosition,
		const wxSize &size = wxDefaultSize,
		long style = wxPALETTEFRM_DEFAULT_STYLE,
		const wxString &name = wxT("mgGreekPalette")) :
		wxPaletteFrame(parent, id, title, pos, size, style, name) {

		// build everything...
		Build(client ? client : parent, mathwnd, mathbox);
	}

	virtual ~mgGreekPalette() {}
};



#endif	// GREEKDLG_H

