///////////////////////////////////////////////////////////////////////
// MathGUI = a WYSIWYG equation editor + a powerful math engine      //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MathBox.h
//! \brief  Defines the mgMathBox and the mgResizeableMathBox classes.
//!
//! See MathBox.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef MATHBOX_H
#define MATHBOX_H

// includes
#include "mc/Element.h"
#include "mc/Text.h"
#include "mc/MathLine.h"
#include "mc/MathOrSystem.h"
#include <wx/caret.h>

// required because mgMathBox uses a WX extension written by Markus Greither
#include "wx/resizec.h"
#include "wx/keybinder.h"


// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MathBox.h"
#endif



//! \name Context menu IDs
//! Some definitions about the IDs used in the context menu of a mgMathBox. @{

//! The first ID reserved to context menu commands.
#define mgMATHBOX_CONTEXTMENU_BASEID			mgMATHBOX_BASEID

//! The first ID reserved to context menu MATH commands.
#define mgMATHBOX_CONTEXTMENU_MATHCMD_BASEID	mgMATHBOX_CONTEXTMENU_BASEID+20

//! Maximum number of items contained in context menu.
#define mgMATHBOX_CONTEXTMENU_FINALID			mgMATHBOX_CONTEXTMENU_BASEID+32

//@}



// the mgMathBox class is able to send to the wxEvtHandler attached through
// the mgMathBox::SetClientHandler, some specific events:
BEGIN_DECLARE_EVENT_TYPES()
	DECLARE_EVENT_TYPE( mgEVT_ACTIVE_ELEM_CHANGED, mgMATHBOX_EVENTS_BASEID )
END_DECLARE_EVENT_TYPES()


//! Adds an event handler for the mgEVT_ACTIVE_ELEM_CHANGED custom event:
//! the wxEvtHandler registered as client event handler of the mgMathBox
//! window will receive such an event each time the element highlighted
//! by mouse cursor changes.
#define EVT_ACTIVE_ELEM_CHANGED(winid, fn)							\
	DECLARE_EVENT_TABLE_ENTRY( mgEVT_ACTIVE_ELEM_CHANGED, winid,	\
		wxID_ANY, (wxObjectEventFunction) (wxEventFunction)			\
		wxStaticCastEvent( wxCommandEventFunction, & fn ),			\
		(wxObject *) NULL ),



//! A little class used to bypass some wxWidget's caret
//! features which are not useful to mgMathBox.
class mgCaret : public wxCaret
{
public:
	mgCaret(wxWindow *window, int w, int h)
		: wxCaret(window, w, h) {}

	//! Shows the caret bypassing standard wxCaret behaviour which is:
	//! if the caret was hidden N times, it must be shown N times 
	//! as well to reappear on the screen.
	//! This function always shows the caret, even if Hide() or
	//! ForceHide() have been called one million times before.
	void ForceShow()	{ DoShow(); m_countVisible=1; } 

	//! Like #ForceShow() but this function hides the caret.
	void ForceHide()	{ DoHide(); m_countVisible=0; } 
};



//! Possible command ids.
//! These are the values you can pass to the #ExecCmd() function.
enum mgMathBoxCmdID {

	// standard edit commands
	mgMBC_CUT = 0,
	mgMBC_COPY_ASPLAIN,
	mgMBC_COPY_ASMATHML,
	mgMBC_PASTE,
	mgMBC_SELECTALL,

	mgMBC_CREATENEW_ANDLINE,
	mgMBC_CREATENEW_ORLINE,
	mgMBC_SIMPLEEND,

	// advanced edit commands
	mgMBC_EMBED_NUMFRACTION,
	mgMBC_EMBED_DENFRACTION,
	mgMBC_EMBED_RADICAL,
	mgMBC_EMBED_BRACKET,
	mgMBC_ADVEND,

#ifdef __MGDEBUG__
	mgMBC_DEBUG,
#endif

	mgMBC_LAST
};

//! A MATH+GUI command which is handled by mcMathMng.
struct mgMathBoxCmd {
	mgMathBoxCmdID m_nID;
	const wxChar *m_strDescription;
};




//! A window using a mcMathOrSystem to create the ouput.
//! mgMathBox class works as a normal wxWindow; so you can
//! allocate, delete or copy mgMathBox windows without problems.
//!
//! mgMathBox can be considered as a "math text control": a control
//! where math expressions can be typed using all MathCore and MathGUI
//! facilities. 
//!
//! Through EnableXXXX() and DisableXXXX() you can modify
//! the behaviour of the window.
//!
class mgMathBox : public wxWindow
{
protected:		// member variables
	
	//! The mcMathOrSystem object contained in this box.
	mcMathOrSystem m_mathContent;

	//! The current position of the mcMathOrSystem in client coordinates.
	wxPoint m_ptPos;

	//! The current cursor for this window.
	//! This value is affected mainly by:
	//! 1) the #m_bInputEnabled variable which forces the window to
	//!    the standard cursor instead of the IBEAM one
	//! 2) the selection system status: when dragging a selection
	//!    rectangle, the cursor is changed to the standard one.
	//wxCursor *m_pCursor;

	//! The last selection rectangle.
	wxRect m_rcSelection;

	//! The last selection.
	mcElement m_eSelection;

	//! The last list of the ID commands of the context menu.
	//mcMathMngCmdID m_nCmdID[mgMATHBOX_MAXMENU_ITEMS];

	//! The event handler which receives the mgEVT_ACTIVE_ELEM_CHANGED
	//! custom events.
	wxEvtHandler *m_pClient;


protected:

	//! If TRUE, we are processing a special character: that is,
	//! a character which must be considered as mapped with the
	//! 'special' font of mcStyle...
	bool m_bProcSpecialChar;

	//! If TRUE, this window automatically expands when the math
	//! content becomes too big.
	bool m_bAutoExpand;


	//! If FALSE, this window will refuse any keyboard input and
	//! the caret will not be displayed. Otherwise, normal input
	//! behaviour is used.
	bool m_bInputEnabled;

	//! If FALSE, the user won't be able to draw selection rects
	//! and thus he won't be able to select anything in the window.
	bool m_bSelectionEnabled;

	//! If FALSE, the user won't be able to popup the context menu
	//! and thus he won't be able to directly modify the math
	//! contents (even if an external function can always call
	//! #GetMathObj()->MATH_FUNCTION to perform some operation...).
	bool m_bContextMenuEnabled;

	// This enables/disables preprocessing of 
	bool m_bPreProcessingEnabled;


protected:			// internal utilities

	//! Inits the window's own variables. Must be called by constructors only.
	void Init();

#ifdef __WXMSW__		// WXMSG is not defined in all ports

	// In the wxMSW port, this function determines if the messages
	// should be preprocessed (by acceleration tables, for example) or
	// they should just be sent to the the event handler of this window
	// "as is": by default, wxWindow::MSWShouldPreProcessMessage returns
	// TRUE but mgMathBox and 
	virtual bool MSWShouldPreProcessMessage(WXMSG* pMsg) {
		return m_bPreProcessingEnabled;
	}

#endif

	//! Returns the caret associated with this window (can be NULL)
	mgCaret *GetCaret() const		{ return (mgCaret *)wxWindow::GetCaret(); }

public:				// customizable variables

	//! Space between the math systems and the left border
	static int nSpaceAround;

	//! Cursor width (in pixels).
	static int nCursorWidth;

	//! If TRUE, then all the strings exported by the IO functions
	//! and by the GUI functions which output strings in the clipboard,
	//! will contain newlines encoded in the native format of the OS
	//! this program is running on.
	//! If FALSE, all strings exported will contain newlines as "\n"
	//! (which are not correctly recognized as newlines under win32 and mac).
	static bool bUseNativeNewLines;

	//! The list of the commands (ID+description) shown in the context
	//! menu. These commands are usually called also from menus
	//! or toolbars, and this variable can be very useful in this case
	//! to synchronize toolbar & menu items' descriptions with the context
	//! menu items.
	static mgMathBoxCmd cmd[mgMBC_LAST+1];

public:

	//! Default constructor. Builds and initialize this window.
	mgMathBox(wxWindow *parent,
		wxWindowID id = -1,
		const wxPoint &pos = wxDefaultPosition,
		const wxSize &size = wxDefaultSize,
		long style = wxSUNKEN_BORDER 
		| wxNO_FULL_REPAINT_ON_RESIZE, 
		const wxString &name = wxT("mgMathBox"))
		: wxWindow(parent, id, pos,  size, style, name) {
			Init();
	}

	//! Copy constructor.
	//! Deep copies the given mgMathBox into this object.
	//! \param tocopy The box to copy.
	mgMathBox(const mgMathBox &tocopy) { DeepCopy(tocopy); }

	//! Destructor.
	//! Destroys everything associated with this window, also math data.
	virtual ~mgMathBox() {
		// the SetCaret() function will delete the caret, if it exists...
		SetCaret(NULL);
	}


public:

	/////////////////////////////////////////////////////////////////////////////////
	//! \name Miscellaneous functions
	//! Some miscellaneous functions. @{

	//! Copies in this object all the data of the given mathbox. The
	//! mathdata associated with this object after a deep copy will
	//! be totally independent from original data.
	void DeepCopy(const mgMathBox &);

	//! Returns a pointer to the contained mcMathOrSystem.
	mcMathOrSystem &GetMathObj()					{ return m_mathContent; }
	const mcMathOrSystem &GetMathObj() const		{ return m_mathContent; }

	//! Enables/disables the keyboard input for this window (by default, it's on).
	void EnableInput(bool bEnable = TRUE);

	//! Enables/disables the selection for this window (by default, it's on).
	void EnableSelection(bool bEnable = TRUE) {
		m_bSelectionEnabled = bEnable;
	}

	//! Enables/disables the context menu for this window (by default, it's on).
	void EnableContextMenu(bool bEnable = TRUE) {
		m_bContextMenuEnabled = bEnable;
	}

	//! Enables/disables the preprocessing of messages under Win32.
	void EnablePreProcessing(bool bEnable = TRUE) {
		m_bPreProcessingEnabled = bEnable;
	}

	void DisableInput() { EnableInput(FALSE); }
	void DisableSelection() { EnableSelection(FALSE); }
	void DisableContextMenu() { EnableContextMenu(FALSE); }
	void DisablePreProcessing() { EnablePreProcessing(FALSE); }

	//! Sets the client event handler which will receive the
	//! mgEVT_ACTIVE_ELEM_CHANGED notification events.
	//! By default, client event handler is set, by constructor,
	//! as the parent window.
	//! Use NULL to disable mgEVT_ACTIVE_ELEM_CHANGED notification
	//! events.
	void SetClientEvtHandler(wxEvtHandler *p) {
		m_pClient = p;
	}

	//! Returns the currently active element (that is, the element which
	//! is currently under the mouse cursor).
	mcElement GetActiveElem(const wxPoint &pos, const wxPoint &pt) const {
		return m_mathContent.gui_GetActiveElem(pos.x, pos.y, pt);
	}

	//! Returns the element with the given ID or NULL if there is no
	//! element with such an ID in this box.
	mcElement GetElemFromID(int id) const {
		return m_mathContent.data_GetElemFromID(id);
	}

	//! Returns a string with debug info about this window.
	wxString GetDebug() const;

	//@}



	/////////////////////////////////////////////////////////////////////////////////////
	//! \name GUI functions regarding SIZE and POSITION
	//! Functions which make graphical input/output easier; most of these
	//! have the same name and behave like mcElementGUI functions. @{

	//! Updates the size of the window adapting it to the size of the contained
	//! math data, plus some corner space around it.
	void FitContents();

	//! Sets the size hints for this window.
	//! Setting "size hints" means that, after this call, this window
	//! will not be able to have a size smaller than the contents size.
	void SetSizeHintsToFitContents();

	//! Returns the size of the mcMathOrSystem contained with also the empty space
	//! which is left by mgMathBox around it.
	wxSize GetContentSize() const;

	//! Deletes all the data from the window: data lines, bitmaps and everything else;
	//! then, adds a new empty line and selects it.
	void Clean();

	//! Returns the x-offset of the contained math data.
	int GetXPos() const		{ return m_ptPos.x; }

	//! Returns the y-offset of the contained math data.
	int GetYPos() const		{ return m_ptPos.y; }

	//! Returns a wxRect initialized with the math content's bounding box
	wxRect GetRectOfContents()
		{ return wxRect(wxPoint(GetXPos(), GetYPos()), GetContentSize()); }

	//@}



	/////////////////////////////////////////////////////////////////////////////////////
	//! \name UPDATE functions
	//! Functions which update the internal state of this mgMathBox. @{

	//! Updates the bitmap in the mcMathLine structure for the n-th line, using
	//! the given flags (and eventually the given point object).
	//! If mcMathSystem::gui_NeedsRefresh() returns TRUE after the update, then
	//! the function redraws the math bitmap (without erasing the background
	//! or calling the Refresh() function to avoid flickering).
	//! \param flags The flags given to the mcMathSystem::gui_UpdateBmp function.
	//! \param p A valid wxPoint class, if you want to save in the
	//!	       bitmap also an eventual activation state.
	//! \return The ID of the active element.
	int UpdateMathBmp(long flags, const wxPoint &p = wxDefaultPosition);

	//! Updates the math content's position using the current client size.
	//! This function alters the #m_ptPos variable.
	void UpdateContentPos();

	//! Updates the cursor position after a change in the size of the selected line;
	//! you should refresh the window after a call to this function, or, at least,
	//! redraw the selected line.
	void UpdateCaretPos();

	//! Updates the caret of this window, eventually showing/hiding it.
	void UpdateCaret();

	//! Updates the bitmap in the mcMathLine structure for the n-th line, using
	//! the current mouse position as argument for #UpdateMathBmp.
	void UpdateMathBmpWithCurrentPos();

	//! Forces the update of the entire math contents.
	//! This function should not be used to often if you want to avoid flickering.
	void ForceBmpUpdate();

	//@}

	


	/////////////////////////////////////////////////////////////////////////////////
	//! \name Miscellaneous
	//! . @{

	//! Redraws the math contents on the screen, after hiding the caret.
	//! This function automatically calls UpdateCaret() before exiting,
	//! so that the caret is restored, if necessary.
	//! Since drawing could be corrupted if done when the caret is blinking
	//! in the blit area, the direct call
	//!          m_mathContent.gui_DrawBmp(....);
	//! should NEVER be used.
	void RedrawMath(wxDC &dc, int x, int y);

	//! Returns the context menu with all the available commands for the
	//! currently selected element, which is given.
	void GetContextMenu(wxMenu &, const mcElement &selected) const;

	//! Checks the return code of the mcElement::MoveCursor function.
	//! Essentially, this function deselects the math expression if an user
	//! clicked on an area not containing any element.
	void CheckReturnFlag(int res);

	//! Resets all the GUI of the mcMathOrSystem.
	void ResetGUI();

	//! Returns TRUE if the user is dragging a selection rectangle
	bool isSelecting() const
		{ return m_rcSelection.width != -1; }

	//! Resets the selection rectangle to a NULL rect.
	void ResetSelectionRect() {
		m_rcSelection.x = m_rcSelection.y = -1;
		m_rcSelection.width = m_rcSelection.height = -1;
	}

	//! Deselects the contained math data.
	void DeSelect()	
		{ m_mathContent.gui_DeSelect();	}

	//! Changes the array of styles used by this window.
	void SetStyleArray(mcStyleArray *p) {
		m_mathContent.gui_SetStyleArray(p);
		m_mathContent.gui_DeepRecalcSize();
		ResetGUI();
	}

	//@}




	/////////////////////////////////////////////////////////////////////////////////
	//! \name Context menu handlers
	//! Various functions which are born to handle context menu commands but
	//! can be safely called also from external functions which handle,
	//! for example, toolbars or menus... @{

	void InsertFraction();
	void InsertRadical();
	void InsertFunction();
	void EditExponent();
	void EditSubscript();

	void Cut(const mcElement &sel = mcEmptyElement);
	void CopyAsPlainText(const mcElement &sel = mcEmptyElement);
	void CopyAsMathML(const mcElement &sel = mcEmptyElement);
	void Paste(const mcElement &sel = mcEmptyElement);

	void EmbedAsFractionNum(const mcElement &sel = mcEmptyElement);
	void EmbedAsFractionDen(const mcElement &sel = mcEmptyElement);
	void EmbedAsRadical(const mcElement &sel = mcEmptyElement);

	//@}




	/////////////////////////////////////////////////////////////////////////////////
	//! \name wxWindow's EVENT HANDLERS
	//! Functions linked with wxWindow's events. @{

	void OnSetFocus(wxFocusEvent &);		// focus events
	void OnFocusLost(wxFocusEvent &);

	void OnTimer(wxTimerEvent &);			// miscellaneous events
	void OnPaint(wxPaintEvent &);
	void OnSize(wxSizeEvent &);

	void OnLDown(wxMouseEvent &);			// mouse events
	void OnLUp(wxMouseEvent &);
	void OnRUp(wxMouseEvent &);
	void OnMouseMove(wxMouseEvent &);
	void OnSetCursor(wxSetCursorEvent &);
	void OnLeaveWindow(wxMouseEvent &);

	void OnCtxMenuItem(wxCommandEvent &);	// menu events

	void OnChar(wxKeyEvent &);				// keyboard events
	void OnSpecialChar(wxKeyEvent &);

	//@}




	/////////////////////////////////////////////////////////////////////////////////
	//! \name Line management functions
	//! These are the functions used by mgMathBox to handle mcMathLine structures. @{

/*	//! Adds one "line" to the mcMathOrSystem contained.
	//! The new "line" (which is an euqation/inequality in the math system),
	//! will be copied from the last one if bCopyLast == TRUE.
	void AddLine(bool bCopyLast);

	//! Deletes the n-th line from the contained mcMathOrSystem.
	void DeleteLine(int n);

	//! Selects the n-th line of the contained mcMathOrSystem.
	void SelectLine(int n)	{ m_mathContent.gui_SelectLine(n); }

	//! Returns the ID of the selected line.
	int GetSelLineId();*/

	void AppendEditCommands(wxMenu &);

	//@}




	/////////////////////////////////////////////////////////////////////////////
	//! \name IO functions
	//! Functions which are used to import/export the math data. @{

	//! Just choose a filename and a title for a new page, and this
	//! function will magically create a complete file containing all
	//! the mathematical data currently displayed in this window.
	//!
	//! Because of the lack of a MathML implementation on Internet Explorer,
	//! this function will also inform the user, if the program is running on
	//! a Win32 system, that he will need an external program to view the
	//! file with IE: a link to MathPlayer homepage will be displayed.
	//!
	//! This function can export the data both as XHTML and as XML. The
	//! differences between the two files are just on the headers, but the
	//! XML file needs some external XSL files (the XML file that is created
	//! is linked with the XSL files of MathStudio 'XSL' subfolder with
	//! absolute path link).
	//!
	//! \param bXHTML    If TRUE, an XHTML is exported; otherwise a pure XML file.
	//! \param filename  The name of a new XHTML/XML file that will be created.
	//! \param title     The title of the new XHTML/XML file.
	//! \param bUpperTag If TRUE, tags are exported in UPPERCASE.
	//! \param bMPSupp   If TRUE, then math player support is enabled,
	//!                  adding some specific tags...
	//! \param bUseNS    If TRUE, then a namespace is used for MathML tags.
	//! \param ns        If bUseNS == TRUE, then this is the namespace used.
	//! \param href
	//! \param indstep   NOT WORKING
	//!
	//! \return TRUE if everything went okay. FALSE if something was wrong;
	//!	        if something was wrong, the function will also advise the
	//!	        user with a message box.
	bool Export(bool bXHTML, const wxString &filename, const wxString &title,
		bool bUpperTag = FALSE, bool bMPSupp = FALSE, bool bUseNS = FALSE,
		const wxString &ns = wxEmptyString, const wxString &href = wxEmptyString,
		int indstep = 2) const;

	//! Exports the math data and works exactly as the other #Export overload
	//! but it accepts a wxOutputStream instead of a file name.
	//! \param str  The output stream where all the XHTML/XML data is copied.
	bool Export(bool bXHTML, wxOutputStream &str, const wxString &title,
		bool bUpperTag = FALSE, bool bMPSupp = FALSE, bool bUseNS = FALSE,
		const wxString &ns = wxEmptyString, const wxString &href = wxEmptyString,
		int indstep = 2) const;

	//! Returns an (usually) huge wxString containing the exported XHTML/XML
	//! math data. It accepts the same parameters as the #Export function.
	wxString GetExportPreview(bool bXHTML, const wxString &title,
		bool bUpperTag = FALSE, bool bMathPlayerSupport = FALSE,
		bool bUseNS = FALSE, const wxString &ns = wxEmptyString,
		const wxString &href = wxEmptyString, int indentstep = 2) const;

	//! Returns the root node associated with the mcMathOrSystem contained in
	//! this box. For more info see mcMathOrSystem::GetMathML().
	wxXml2Node GetMathML(bool bGetPresentation) const {
		return m_mathContent.io_GetMathML(bGetPresentation);
	}

	//! Gets the inlined expression for the math contents.
	wxString GetInlinedExpr() const {
		return m_mathContent.io_GetInlinedExpr();
	}

	//! Imports XHTML/XML math data.
	bool Import(int type, const wxString &filename, wxString *pDesc = NULL);

	//! Imports MathML already parsed, eventually returning the error
	//! description (if an errors occurs, the function returns FALSE).
	bool ImportMathML(wxXml2Node pDoc, wxString *pErr) {
		return m_mathContent.io_ImportPresentationMathML(pDoc, *pErr);
	}

	//! Imports the given inlined expression.
	bool ImportInlinedExpr(const wxString &str);

	//! A little utility function which converts "\n" as the native
	//! character/pair of characters used to represent newlines
	//! (that is "\r\n" on win32), if #bUseNativeNewLines is TRUE.
	wxString ConvertString(const wxString &str) const;

	//@}


private:
	DECLARE_CLASS(mgMathBox)
	DECLARE_EVENT_TABLE()
};




#if 1

//! Base class for a resizeable box which can be contained inside a mgMathWnd.
//! VERY IMPORTANT:
//! \li In your contructor, you must do two required steps, in order to avoid
//!     weird effects: create your wrapped window and, as last operation, call
//!     the #SetMinBorder function to update your child window and show it in
//!     the right place of the resizeable box.
//! \li If you implement an event-handler for a mgResizeableBox-derived class,
//!     you must recall the mgResizeableBox version of that event-handler.
//!
class mgResizeableBox : public wxResizeableControl
{
protected:

	//! The minimum border.
	int m_nMinBorder;

	//bool m_bFocus, m_bChildFocus;

public:
	//! Main constructor. Builds and initialize this window.
	//! \param parent The parent window
	mgResizeableBox(wxWindow *parent,
		wxWindowID id = -1,
		const wxPoint &pos = wxDefaultPosition,
		const wxSize &size = wxDefaultSize);

	//! Default constructor. This is required by wxWidgets
	//! DECLARE_DYNAMIC_CLASS macro, but you should always use
	//! the main constructor form.
	mgResizeableBox() {
		m_nMinBorder = 0;
	}

	//! Copy constructor. Deep copies the given mgMathBox into this object.
	//! \param tocopy The box to copy
	mgResizeableBox(const mgResizeableBox &tocopy);

	//! Destructor.
	virtual ~mgResizeableBox() {
		//GetParent()->GetParent()->SetFocus();
	}

	//! Copies all the info contained in the given resizeable math box.
	void DeepCopy(const mgResizeableBox &);

	//! Sets the minimum number of pixels which compose the border of this
	//! resizeable box. If the user tries to make this window smaller than
	//! the minimum border + the contained math box's minimum size, this
	//! window will adjust the resize request to the smallest available size.
	void SetMinBorder(int n);

	//! Returns the minimum border always present aroung the contained math box.
	//! That minimum border is where the 8 squares which can be dragged for
	//! resizing, are placed.
	int GetBorder();

	//void SetFocus();
	//void Refresh(bool eraseBackground = TRUE, const wxRect* rect = NULL);

	//! Returns a string containing debug information.
	virtual wxString GetDebug();


	//! \name 'Abstract' functions
	//! The quotes around the word 'abstract' mean that these functions are not
	//! declared as C++ pure virtual functions: they cannot be used if you are
	//! using the DECLARE_DYNAMIC_CLASS wxWidgets macro. @{

	virtual wxWindow *GetChild()			{ return NULL; }

	//@}

	// event handlers
	virtual void OnSize(wxSizeEvent &);
	virtual void OnSetFocus(wxFocusEvent &);
	virtual void OnKillFocus(wxFocusEvent &);
	virtual void OnSetCursor(wxSetCursorEvent &ev);
	virtual void OnPaint(wxPaintEvent &ev);
	virtual void OnMove(wxMoveEvent &ev);

	virtual void OnChildKillFocus(wxCommandEvent &);
	virtual void OnChildSetFocus(wxCommandEvent &);
	virtual void OnChildResize(wxCommandEvent &);


private:
	DECLARE_CLASS(mgResizeableBox)
	DECLARE_EVENT_TABLE()
};
/*
class mgResizeableBox2 : public wxResizeableHub
{
protected:

	//! The minimum border.
	int m_nMinBorder;

	//bool m_bFocus, m_bChildFocus;

public:
	//! Main constructor. Builds and initialize this window.
	//! \param parent The parent window
	mgResizeableBox(wxWindow *parent,
		wxWindowID id = -1,
		const wxPoint &pos = wxDefaultPosition,
		const wxSize &size = wxDefaultSize);

	//! Default constructor. This is required by wxWidgets
	//! DECLARE_DYNAMIC_CLASS macro, but you should always use
	//! the main constructor form.
	mgResizeableBox(); {
		m_nMinBorder = 0;
	}

	//! Copy constructor. Deep copies the given mgMathBox into this object.
	//! \param tocopy The box to copy
	mgResizeableBox(const mgResizeableBox &tocopy);

	//! Destructor.
	virtual ~mgResizeableBox() {
		//GetParent()->GetParent()->SetFocus();
	}

	//! Copies all the info contained in the given resizeable math box.
	void DeepCopy(const mgResizeableBox &);

	//! Sets the minimum number of pixels which compose the border of this
	//! resizeable box. If the user tries to make this window smaller than
	//! the minimum border + the contained math box's minimum size, this
	//! window will adjust the resize request to the smallest available size.
	void SetMinBorder(int n);

	//! Returns the minimum border always present aroung the contained math box.
	//! That minimum border is where the 8 squares which can be dragged for
	//! resizing, are placed.
	int GetBorder();

	void SetFocus();
	//void Refresh(bool eraseBackground = TRUE, const wxRect* rect = NULL);

	//! Returns a string containing debug information.
	virtual wxString GetDebug();


	//! \name 'Abstract' functions
	//! The quotes around the word 'abstract' mean that these functions are not
	//! declared as C++ pure virtual functions: they cannot be used if you are
	//! using the DECLARE_DYNAMIC_CLASS wxWidgets macro. @{

	virtual wxWindow *GetChild()			{ return NULL; }

	//@}

	// event handlers
	virtual void OnSize(wxSizeEvent &);
	virtual void OnSetFocus(wxFocusEvent &);
	virtual void OnKillFocus(wxFocusEvent &);
	virtual void OnSetCursor(wxSetCursorEvent &ev);
	virtual void OnPaint(wxPaintEvent &ev);

	virtual void OnChildKillFocus(wxCommandEvent &);
	virtual void OnChildSetFocus(wxCommandEvent &);
	virtual void OnChildResize(wxCommandEvent &);


private:
	DECLARE_CLASS(mgResizeableBox2)
	DECLARE_EVENT_TABLE()
};
*/
//! A resizeable window containing a mgMathBox. The resizeable window appears (by default)
//! as a yellow border around the wrapped mgMathBox, with 8 squares which can be
//! dragged by the user to resize the window. The empty border can be used to move the
//! window (typically inside a mgMathWnd which acts as a mgResizeableMathBox container).
//!
//! You can change the colour of the window using wxWindow::SetBackgroundColor()
//! but remember that the background colour determines also the square's colour;
//! infact the squares are drawn with the wxXOR logical function.
//!
//! You can change the thickness of the border around the mgMathBox using the
//! #SetMinBorder function.
class mgResizeableMathBox : public mgResizeableBox
{
protected:

	//! The contained math box.
	mgMathBox *m_pMathBox;//m_mathBox;

public:
	//! Main constructor. Builds and initialize this window.
	//! \param parent The parent window
	mgResizeableMathBox(wxWindow *parent,
		wxWindowID id = -1,
		const wxPoint &pos = wxDefaultPosition,
		const wxSize &size = wxDefaultSize);

	//! Default constructor. This is required by wxWidgets
	//! DECLARE_DYNAMIC_CLASS macro, but you should always use
	//! the main constructor form.
	mgResizeableMathBox() {};// : m_mathBox() {}

	//! Copy constructor. Deep copies the given mgMathBox into this object.
	//! \param tocopy The box to copy
	mgResizeableMathBox(const mgResizeableMathBox &tocopy);

	//! Destructor.
	virtual ~mgResizeableMathBox() {
		if (m_pMathBox) delete m_pMathBox;
	}

	//! Copies all the info contained in the given resizeable math box.
	void DeepCopy(const mgResizeableMathBox &);

	//! Returns a string with debug info about this window.
	wxString GetDebug();

	//! Returns an XML tree containing the MathML code representing
	//! the math data stored in the math box
	virtual wxXml2Node GetMathML(bool bGetPresentation) {
		return m_pMathBox->GetMathML(bGetPresentation);
	}

	//! Imports MathML inside this resizeable box.
	virtual bool ImportMathML(wxXml2Node pDoc, wxString *pErr) {
		return m_pMathBox->ImportMathML(pDoc, pErr);
	}


	wxWindow *GetChild()			{ return m_pMathBox; }//&m_mathBox; }

	//! \name INLINE mathline functions.
	//! Functions which recall directly mgMathBox's functions. @{
/*	void DeleteLine(int n)			{ m_pMathBox->DeleteLine(n); }
	void SelectLine(int n)			{ m_pMathBox->SelectLine(n); }
	void AddLine(bool bCopyLast)	{ m_pMathBox->AddLine(bCopyLast); }
	int GetSelLineId()				{ return m_pMathBox->GetSelLineId(); }*/
	//@}

	// event handlers
	//virtual void OnSize(wxSizeEvent &);

private:
	DECLARE_CLASS(mgResizeableMathBox)
	DECLARE_EVENT_TABLE()
};



//! A resizeable box which contains text. This window appears exactly like
//! a wxTextCtrl (infact all the text-input stuff is dropped to a wxTextCtrl),
//! with just 8 squares which can be used to resize the window.
class mgResizeableTextBox : public /*wxTextCtrl, public*/ mgResizeableBox
{
protected:

	//! The contained text.
	wxTextCtrl *m_pTextCtrl;
	//mcText m_text;

public:
	//! Main constructor. Builds and initialize this window.
	//! \param parent The parent window
	mgResizeableTextBox(wxWindow *parent,
		wxWindowID id = -1,
		const wxPoint &pos = wxDefaultPosition,
		const wxSize &size = wxDefaultSize);

	//! Default constructor. This is required by wxWidgets
	//! DECLARE_DYNAMIC_CLASS macro, but you should always use
	//! the main constructor form.
	//mgResizeableTextBox() : m_pTextCtrl(NULL) {}

	//! Copy constructor. Deep copies the given mgMathBox into this object.
	//! \param tocopy The box to copy
	//mgResizeableTextBox(const mgResizeableTextBox &tocopy);

	//! Destructor.
	virtual ~mgResizeableTextBox() {
		//if (m_pTextCtrl) delete m_pTextCtrl;
	}

	//! Copies all the info contained in the given resizeable math box.
	//void DeepCopy(const mgResizeableTextBox &);

	//! Returns a string with debug info about this window.
	wxString GetDebug();


	wxWindow *GetChild()			{ return m_pTextCtrl; }

	// event handlers
	/*void OnSize(wxSizeEvent &);
	void OnSetFocus(wxFocusEvent &);
	void OnSetCursor(wxSetCursorEvent &ev);
	void OnChildResize(wxCommandEvent &);
	void OnPaint(wxPaintEvent &ev);*/

private:
	DECLARE_CLASS(mgResizeableTextBox)
	DECLARE_EVENT_TABLE()
};

#endif
#endif	// MATHBOX_H

