///////////////////////////////////////////////////////////////////////
// MathGUI = a WYSIWYG equation editor + a powerful math engine      //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MathWnd.h
//! \brief  Defines the mgMathWnd class.
//!
//! See MathWnd.cpp for more info
///////////////////////////////////////////////////////////////////////



#ifndef MATHWND_H
#define MATHWND_H

// required because mgMathWnd uses heavily mgMathBoxes
#include "mg/GUIUtils.h"
#include "mg/MathBox.h"


// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MathWnd.h"
#endif

//! The array of math boxes contained in each mgMathWnd
WX_DECLARE_OBJARRAY(mgResizeableBox, wxMathBoxArray);
//WX_DECLARE_OBJARRAY(mgResizeableTextBox, wxTextBoxArray);


//! A container window of text and mgMathBoxes.
class mgMathWnd : public wxScrolledWindow
{
	//! Array of mgMathBox classes.
	wxMathBoxArray m_arrMathBox;
	//wxTextBoxArray m_arrTextBox;
	
	// various
	wxSize m_szClient;				//!< The virtual size of the client area.
	int m_nSelBox;					//!< The selected line inside the array.
	int m_nActiveElemID; 		//!< The ID of the element currently under cursor.
	
	//! The memory DC used to blit bitmaps; at the end of each mgMathBox call,
	//! no bitmaps should be selected in this DC.
	wxMemoryDC m_hMemDC;
	
	//! The point where the user started the selection.
	wxPoint m_ptOrigin;
	
	//! The last selection rectangle.
	wxRect m_rcSelection;
	
	//! The last list of the ID commands of the context menu.
	//int m_nCmdID[MAXMENU_ITEMS];
	
public:
	
	//! Constructor. Builds and initializes a mgMathWnd window.
	mgMathWnd(wxWindow *parent, 
		wxWindowID id = -1, 
		const wxPoint &pos = wxPoint(0, 0));
	
	//! Destructor. Destroys everything associated with this window.
	virtual ~mgMathWnd();
	

	

	//! Recalculates the scrollbars
    void UpdateScrollRange();

	//! Deletes all the data from the window: data lines, bitmaps and everything else;
	//! then, adds a new empty line and selects it.
	void Clean();

	//! Returns a string with debug info about this window.
	wxString GetDebug();

	//! Changes the style used by all the boxes contained in this window.
	void SetStyleArray(mcStyleArray *) {}
	
	//! Just choose a filename and a title for a new page, and this 
	//! function will magically create a complete file containing all the
	//! mathematical data currently displayed in this window.
	//! Because of the lack of a MathML implementation on Internet Explorer,
	//! this function will also inform the user, if the program is running on
	//! a Win32 system, of the program he will need to view the file with IE.
	//! This function can export the data both as XHTML and as XML. The
	//! differences between the two files are just on the headers, but the
	//! XML file needs some external XSL files (the XML file that is created
	//! is linked with the XSL files of MathStudio 'XSL' subfolder with
	//! absolute path link).
	//! \param type The type of the file to export; use EXPORTAS_XHTML or
	//!		  EXPORTAS_XML values.
	//! \param filename The name of a new XHTML file that will be created.
	//! \param title The title of the new XHTML file.
	//! \return TRUE if everything went okay. FALSE if something was wrong;
	//!	     if something was wrong, the function will also advise the
	//!	     user with a message box.
	bool Export(bool bXHTML, const wxString &filename, const wxString &title, 
		bool bUpperTag = FALSE, bool bMathPlayerSupport = FALSE, bool bUseNS = FALSE, 
		const wxString &ns = wxEmptyString, const wxString &href = wxEmptyString, 
		int indentstep = 2);

	bool Export(bool bXHTML, wxOutputStream &stream, const wxString &title, bool bPreview, 
		bool bUpperTag = FALSE, bool bMathPlayerSupport = FALSE, bool bUseNS = FALSE, 
		const wxString &ns = wxEmptyString, const wxString &href = wxEmptyString, 
		int indentstep = 2);

	wxString GetExportPreview(bool bXHTML, const wxString &title, 
		bool bUpperTag = FALSE, bool bMathPlayerSupport = FALSE,
		bool bUseNS = FALSE, const wxString &ns = wxEmptyString, 
		const wxString &href = wxEmptyString, int indentstep = 2);
	
	bool Import(int type, const wxString &filename, wxString *pDesc = NULL);
	
	
	

	/////////////////////////////////////////////////////////////////////////////////
	//! \name wxWindow's EVENT HANDLERS
	//! Functions linked with wxWindow's events. @{

	void OnSetFocus(wxFocusEvent &);		// focus event
	void OnFocusLost(wxFocusEvent &);
	
	void OnTimer(wxTimerEvent &);			// miscellaneous events
	void OnPaint(wxPaintEvent &);
	
	void OnLDown(wxMouseEvent &);			// mouse events
	void OnRDown(wxMouseEvent &);
	void OnLUp(wxMouseEvent &);
	void OnRUp(wxMouseEvent &);
	void OnMouseMove(wxMouseEvent &);
	
	void OnCtxMenuItem(wxCommandEvent &);
	
	void OnChar(wxKeyEvent &);				// keyboard events
	void OnSize(wxSizeEvent &);

    //! Responds to child window notification
    void OnChildWindowChange(wxCommandEvent &);

	//@}
	
	
	

	/////////////////////////////////////////////////////////////////////////////////
	//! \name Box management functions
	//! These are the functions used by mgMathWnd to handle the various boxes. @{
	
	//! Selects the given line.
	//! Selected line is drawn with the MathStudio::m_pSelectionBrush as background;
	//! the selected line is the only line the user can modify. Only one line
	//! can be selected at each time.
	//! \param n The line to select
	void SelectBox(int n);
	
	//! Selects the line whose bounding box contains the given point.
	//! \param &pt The Given point must be expressed in scrolled window coord.
	void SelectBoxByPoint(wxPoint &pt);
	
	//! Adds one mgResizeableMathBox or a mgResizeableTextBox to this window.
	//! The function will create, select and, optionally, init the new box with
	//! the content of the previously selected box.
	//! \param bMath If TRUE, a mgResizeableMathBox is added, otherwise a
	//!              mgResizeableTextBox is added.
	//! \param bCopyLast If TRUE, the previously selected line's contents will be
	//!	                 copied in the new line.
	void AddBox(bool bMath, bool bCopyLast);

	//! Remove the n-th bo linked to this window.
	void DeleteBox(int n);
	
	//@}
	

	
	///////////////////////////////////////////////////////////////////////////////////
	//! \name Inline GET functions
	//! Fast functions to retrieve data; possibly use these also in the
	//! other mgMathBox functions. @{
/*	
	int GetSelBoxId()					{ return m_nSelBox; }
	int GetBoxCount()					{ return GetChildren().GetCount(); }
	mgResizeableBox *GetBox(int n) 		{ return (mgResizeableBox *)((GetChildren().Item(n))->GetData()); }
	mgResizeableBox *GetSelBox()			{ return GetBox(m_nSelBox); }
*/
	int GetSelBoxId()					{ return m_nSelBox; }
	int GetBoxCount()					{ return m_arrMathBox.GetCount(); }
	mgResizeableBox *GetBox(int n) 		{ return &(m_arrMathBox.Item(n)); }
	mgResizeableBox *GetSelBox()			{ return GetBox(m_nSelBox); }

	//@}
	
	
	// as wxWindow, an event table is required
	DECLARE_CLASS(mgMathWnd)
	DECLARE_EVENT_TABLE()
};


#endif		// MATHWND_H

