///////////////////////////////////////////////////////////////////////
// MathGUI = a WYSIWYG equation editor + a powerful math engine      //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
// \file   OptBasePanel.h
// \brief  Defines the mgOptBasePanel class.
//
// See OptBasePanel.cpp for more info.
///////////////////////////////////////////////////////////////////////




#ifndef DIALOGS_BASEPANEL_H
#define DIALOGS_BASEPANEL_H


#if defined(__GNUG__) && !defined(__APPLE__)
	#pragma interface "OptBasePanel.h"
#endif

// includes
#include "wx/string.h"
#include "mg/OptDlg.h"
class wxConfig;


/*! \class mgOptBasePanel
 \brief The \c Edit | \c Options dialog box: optionwxT('s panel')s base class.
 \author L. Cattani
 \date 3-06-2003

 This is the base class for option's panels in the dialog box.
 When writing a new option's panel, derive from this class and override the
 \c CommitChanges and \c Setup virtual function.

 Classes derived from this class should only define the default
 constructor that should not do anything.
 In particular, the default ctor should not set window's properties
 such as fonts or background color nor it does not create any
 child window.

 This is because classes derived from mgOptBasePanel are
 instantiated dynamically at run-time using the \c ::wxCreateDynamicObject
 function that accepts a string as a parameter: the string represents
 the class name of the object to be instantiated.

 But that function only accepts one parameter, the class name, so
 you do not have the possibility to pass parameters to the ctor
 of the class.
 In fact, only objects with the default ctor can be instantiated
 dynamically.

 As a conseguence, the default ctor of the panel calls the default
 ctor of wxWindow class and derivates. The default ctor of the
 wxWindow's windows objects inits their internal structure but
 does not create the underlying GUI structures (that is window handle
 and GTK widgets) so that it is not possibile to set properties
 or to create child windows.
 The underlying structures are created with the \c Create function.

 So, the mgOptDlg::CreatePanel function, which is responsible
 to create the \c mgOptBasePanel -derived class from the
 class name constructs the object dynamically (default ctor) then
 calls \c wxPanel::Create to fully create the underlying GUI
 structure and then the virtual \c mgOptBasePanel::Setup function
 in which the programmer inserts code to setup the layout of
 the panel, change window's property, create child controls, etc

 Althrough the dynamic instantation is, by now, not used, I want to
 used this organization: maybe in the future I will used it.

 To add a new panel in the mgOptDlg class you have to:

 \li derive a new class from mgOptionsBanePanel and override the
 \c Setup and \c CommitChanges functions
 \li add a item in the mgOptionsTreeCtrl constructor in order to
 add the relevant item in the tree list control:

 For example, to add the MySettings panel in the dialog do:

 \code
   mgOptionsTreeCtrl.cpp (ctor)
   wxTreeItemId newId = AppendItem(
         idRoot,        // the parent item of this item (root if applicable)
         wxT("MySettings"),  // the label of the item, passed as parameter in
                        // mgOptBasePanel::Setup() function
         -1, -1,        // icons for tree nodes
         new mgOptTreeCtrlItemData(          // the item-data structure
         Id_MySettingsClass ) );            // the class ID of the panel
 \endcode

 then write your specialized \c MySettingsClassName class derived from
 this class
*/
class mgOptBasePanel : public wxPanel
{
public:
	//! Constructor - it does nothing at all.
	mgOptBasePanel() {}

	//! Destructor - It does nothing at all.
	virtual ~mgOptBasePanel() {}



	//! Commits the changes made by the user.
	/*!
	 The virtual function is called with a pointer to a \c wxConfig object
	 by the \c mgOptDlg::CommitChanges function when the user clicks the
	 \e OK or \e Apply button.

	 The function should change the variables used by MathCore and MathGUI
	 in order to reflect changes in the configuration.
	 In other words, if the user changed the font family and / or size
	 this function should delete the font object used by the application
	 and replace it with a new font ehich caracteristics are those specified
	 by the user and store the pointer of the newly created font object
	 in the MathCore library.

	 There is no need for the function to store the settings in the
	 configuration objevt because they will be stored automatically
	 at program exit in CMathApp::OnExit() function.

	 In order to refresh the application apareance when the user changes
	 options settings (such as a font), the function returns a constant
	 value, which is an enum, that represent what aspect of the application
	 is to be refreshed.
	 It is the caller responsability to do the actual work.
 
	 \sa mgOptDlg#CommitChanges mgOptDlg#mgOptChangedType
	*/
	virtual mgOptDlg::mgOptChangedType CommitChanges( wxConfig* config ) = 0;


	//! The alter ego of the constructor.
	/*!
	 This function is the real ctor of the class. It is called from
	 \c mgOptDlg::CreatePanel function just after the dynamic istantiation
	 of the class and after calling \c wxPanel::Create.

	 As you cannot set properies for the panel such as fonts, background
	 color atc. in the class's ctor you have to set them in this function.

	 The \c itemLabel string contains the label of the treelist's item
	 associated with this option's panel.

	 The default implementation of this class just creates a wxStaticBoxSizer
	 equal to the size of the panel and sets its label as the treelist's
	 item's label.
	 Specialized derived classes should construct the child controls and
	 the sizer or layout constarints objects to be bounded to childs.
	*/
	virtual void Setup( const wxString &itemLabel ) = 0;
	

	
private:
	DECLARE_ABSTRACT_CLASS( mgOptBasePanel )
};


#endif          // not defined DIALOGS_BASEPANEL_H


