///////////////////////////////////////////////////////////////////////
// MathGUI = a WYSIWYG equation editor + a powerful math engine      //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   OptDlg.h
//! \brief  Defines the mgOptDlg class.
//!
//! See OptionsDlg.cpp for more info.
///////////////////////////////////////////////////////////////////////




#ifndef DIALOGS_OPTIONSDLG_H
#define DIALOGS_OPTIONSDLG_H


#if defined(__GNUG__) && !defined(__APPLE__)
  #pragma interface "OptDlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
/*#include "wx/wxprec.h"

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

// for all others, include the necessary headers
#ifndef WX_PRECOMP
  #include "wx/wx.h"
#endif
*/

// includes
#include "mg/Setup.h"
#include "mg/GUIUtils.h"
#include "mg/OptTreeCtrl.h"
#include <wx/config.h>

// forward declarations
class mgOptionsBtnPanel;
class mgOptionsTreeCtrl;
class mgOptBasePanel;


// identifiers for child controls
#define OPTIONSDLG_BTN_OK					mgOPTIONSDLG_BASEID+1
#define OPTIONSDLG_BTN_CANCEL				mgOPTIONSDLG_BASEID+2
#define OPTIONSDLG_BTN_APPLY				mgOPTIONSDLG_BASEID+3
#define OPTIONSDLG_TREELIST					mgOPTIONSDLG_BASEID+4
#define OPTIONSDLG_MAX_TREELIST_DEPTH		6


// base ids for the panels
#define OPTIONSDLG_KEYPANEL_BASEID			mgOPTIONSDLG_BASEID+100
#define OPTIONSDLG_FONTPANEL_BASEID			mgOPTIONSDLG_BASEID+200




// The custom wxCommandEvent event used by the Edit | Options
// dialog box to force an application and/or UI updating.
// This event is 'posted' to the application object when the
// user clicks OK or Apply by the mgOptBasePanel-derived class
// in function CommitChanges().
// To add a new event type just add a line in the following macro table
// In the .cpp file you have to add the definition of the event:
//
//    DEFINE_EVENT_TYPE( mgEVT_NEW_EVENT_TYPE )
//
// and then add an entry in the event table definition
//
//   BEGIN_EVENT_TABLE( CMathApp, wxApp )
//       EVT_CUSTOM( mgEVT_NEW_EVENT_TYPE, id, function)
//   END_EVENT_TABLE
//
// You have to declare and define the wxT('function') event handler. 'id' could
// be -1 (or wxID_ALL
//
BEGIN_DECLARE_EVENT_TYPES()
    DECLARE_EVENT_TYPE( mgEVT_OPTIONS_CHANGED, mgOPTIONSDLG_EVENTS_BASEID )
END_DECLARE_EVENT_TYPES()

#define EVT_OPTIONS_CHANGED(func)									\
	DECLARE_EVENT_TABLE_ENTRY( mgEVT_OPTIONS_CHANGED, wxID_ANY,		\
	wxID_ANY, (wxObjectEventFunction) (wxEventFunction)				\
	wxStaticCastEvent( wxCommandEventFunction, & func ),			\
	(wxObject *) NULL ),




/*! \class mgOptDlg
    \brief The \c Edit | \c Options dialog box: m_mainSizer window class.

The option's dialog box is made of:
 \li a wxTreeListCtrl-derived class on the left side in which are listed all
	option's families in a tree hierarchy.
 \li several wxPanel-derived on the top-right side of the dialog in which
	are displayed the option's values for the selected \e item.
	Only one of these panels is visible at any given moment.
 \li a wxPanel-derived class containing the OK, Cancel, Apply buttons on the bottom-right
	side of the dialog

When the user clicks on a item in the tree list, the corresponding \c mgOptBasePanel-
 derived class window containing all applicable options, is displayed on the right side.
The changes in any optionwxT('s value are made by the specialized option')s panel and
not by this class

To use this class is very simple. In the \e Edit | \e Options menu event handler
write something similar to this:

\code
	mgOptDlg* opt( frame, -1, 0 );
	opt->ShowModal();
\endcode

There is no need to test if the user selects OK or Cancel because
it is mgOptDlg's responsability to store the changes permanently and
to redraw the UI in case option's changes affect the UI appearance.
For more info read \ref howworkseditoptions.

\author L. Cattani
\date 8-11-2003
*/
class mgOptDlg : public wxDialog
{
public:
	//! The type of changes made by the user.
	/*!
	   This \b enum is used to let the application know what type of changes
	   the user made in the options panel.
   
	   Changes may affect the UI (for example a font) or the application or
	   the math engine of MathStudio.
	   Instead of redrawing and re-initializing all the application when the
	   user makes a change, each optionwxT('s panel')s \c CommitChanges function
	   returns a mgOptChangedType that is OR'ed with all other return values.
   
	   The main \c CommitChanges function in this object sets up a custom
	   event and stores the mgOptChangedType value in a data member of the
	   event class so that the custom event handler - located in the \c CMathApp
	   object - only resfresh those aspects of the application that should be
	   refreshed.
	*/
	enum mgOptChangedType  {
		OptChanged_None      = 0,	/*!< no changes		*/
			OptChanged_App       = 1,	/*!< application	*/
			OptChanged_MainWnd   = 2,	/*!< main window	*/
			OptChanged_ParamWnd  = 4,	/*!< parameters window	*/
			OptChanged_UnknonwWnd = 8,	/*!< unknows window	*/
			OptChanged_ConstWnd   = 16	/*!< constants window	*/
	};

	/*!
	  \brief Constructor.

	Constructs the Options dialog box. The function simply creates the
	2 panels: the button panel and the treelist panel and the constraints
	objects used to laying out the three panels.
	Option's panel are not created at dialog startup but on the fly when
	the user selectes one item in the tree list.

	For example, if the user selects the \e Key-bindings item in the 
	tree list then the \c mgOptKeyBindingsPanel class is istantiated and
	displayed.

	\param parent the parent window (usually the m_mainSizer frame)
	\param id the window identifier (should be -1)
	\param config the configuration object from which the options settings are
		read from and stored to (not used yet, should be NULL)
	\sa #CreateLayout
	*/
	mgOptDlg( wxWindow* parent, wxWindowID id, wxConfig* config = 0 );

	virtual ~mgOptDlg();




	//! Creates the three child windows and the initial layout.
	/*!
	 This function creates the three \c wxLayoutConstraints objects used to
	 laying out the three child windows: a treelist control and the two panels.
	 The layout constraints for the treelist and the button panel are
	 bounded to their respective windows in this function.
	 If OPTIONSDLG_USE_LAYOUTCONSTRAINTS is not defined, then this function
	 uses sizers to accomplish its task. This behaviour should be the preffered
	 one: wxLayoutConstraints are deprecated in the new versions of wxWin...

	 The layout for the option's panel is not set for the panel because
	 there is no option's panel yet: they are created on request.
	 The layout for the option's panel is stored in the \c m_optionLayout
	 data member and a copy of it is made for every created option's panel
	 in the \c CreatePanel function.
 
	 \param parent the parent window as specified in the ctor

	 \sa #CreatePanel #m_optionLayout
	*/
	void CreateLayout( wxWindow* parent );
	


	//! Creates the option's panel identified by its \e ID.
	/*!
	 The panel is created and constrained with a newly created copy of
	 the \c m_optionLayout object.

	 The function uses a \b switch statement to create the correct
	 mgOptionsBanePanel -derived class that handles the options
	 for the selected item in the treelist control.

	 Returns the pointer of the created panel window or NULL if the
	 panel could not be created.
 
	 \param itemData the wxTreeCtrlItemData -derived class associated with the
 		selected item in the treelist
	 \param itemLabel the label of the selected item in the trelist control
	*/
	mgOptBasePanel* CreatePanel( mgOptTreeCtrlItemData *itemData,
  										const wxString& itemLabel );

	//! Display the option's panel related to the selected item.
	/*!
	 The function hides the actually displayed option's panel and shows
	 the panel associated in the selected item in the treelist control.

	 If the relevant panel class was not yet created, the \c CreatePanel
	 function is called.

	 \param itemId the identifier of the selected item in the treelist control
 		(must be valid otherwise an ASSERT failure occurs).
	*/
	void DisplayPanel( wxTreeItemId id );
	

	//! Commits the changes made by the user in the option's values.
	/*!
	 This function iterates through all itemData structure of the
	 treelist control and obtains the pointer of the mgOptBasePanel
	 -derived class for every option item.

	 The function calls the mgOptBasePanel::CommitChange function
	 of all option's panels which are responsible of doing the actual work.
	*/
	void CommitChanges();


	//! Sets the event handler which will receive the notification events.
	//! By default, the client event handler is set by constructor as
	//! the parent window. Use NULL to disable event notification.
	void SetClientEvtHandler(wxEvtHandler *p) {
		m_pClientEvtHandler = p;
	}
	

	

protected:

	//! The configuration object in which changes are stored (not used).
	wxConfig*   m_config;
	
	//! The OK, Cancel, Apply button's panel.
	mgOptionsBtnPanel*     m_buttonWnd;
	
	//! The treelist control panel.
	mgOptionsTreeCtrl*     m_treeWnd;
	
	//! The actually displayed option's panel.
	wxPanel*              m_actualPanel;
	
	//! A empty panel - just for testing purposes.
	//wxPanel*      m_emptyPanel;
	
	//! The event handler which receives notification events.
	wxEvtHandler *m_pClientEvtHandler;


	
#ifdef OPTIONSDLG_USE_LAYOUTCONSTRAINTS
	
	//! The layout oject for mgOptBasePanel derived option's panel.
	wxLayoutConstraints* m_optionLayout;
	
#else		// use wxSizers
	
	//! The main sizer of the entire dialog box.
	wxBoxSizer *m_mainSizer;
	
	//! The sizer which contains the current active panel.
	wxBoxSizer* m_optionSizer;
	
#endif
	

protected:				// event handlers


	//! The event handler for the \e OK button.
	/*|
	Calls \c CommitChanges and then \c EndModal( wxID_OK )
	*/
	void OnOkButton( wxCommandEvent &ev );
	
	//! The event handler for the \e Cancel button.
	/*!
	Calls \c EndModal( wxID_CANCEL )
	*/
	void OnCancelButton( wxCommandEvent &ev );
	
	//! The event handler for the \e Apply button.
	/*!
	Calls \c CommitChanges.
	*/
	void OnApplyButton( wxCommandEvent &ev );
	
	//! The event handler for a selection in the treelist control.
	/*!
	The function obtains the \e itemID of the selected item in the
	treelist control and calls the \c DisplayItem function
	*/
	void OnItemSelected( wxTreeEvent& ev );
	
private:
	DECLARE_CLASS( mgOptDlg )
	DECLARE_EVENT_TABLE()	
};


#endif          // not defined DIALOGS_OPTIONSDLG_H

