///////////////////////////////////////////////////////////////////////
// MathGUI = a WYSIWYG equation editor + a powerful math engine      //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
// \file  OptFontSettings.h
// \brief Defines the mgOptFontSettingsPanel class.
//
// See OptFontSettings.cpp for more info.
///////////////////////////////////////////////////////////////////////




#ifndef OPTFONTSETTINGS_H
#define OPTFONTSETTINGS_H


#if defined(__GNUG__) && !defined(__APPLE__)
  #pragma interface "OptFontSettingsPanel.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
/*#include "wx/wxprec.h"

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

// for all others, include the necessary headers
#ifndef WX_PRECOMP
  #include "wx/wx.h"
#endif
*/

// includes
#include <wx/fontenum.h>		// for wxFontEnumerator
#include <mg/OptBasePanel.h>
#include <mg/MathBox.h>		// for mgMathBox
#include <mc/MathUtils.h>	// for mcTextSettings


// ids
#define OPTIONSDLG_FONTPANEL_EXPDEPTH			OPTIONSDLG_FONTPANEL_BASEID+1
#define OPTIONSDLG_FONTPANEL_PREVIEWBOX			OPTIONSDLG_FONTPANEL_BASEID+2

// the base ids for the font-setting rows
#define OPTIONSDLG_FONTPANEL_SETTINGBASEID		OPTIONSDLG_FONTPANEL_BASEID+5
#define OPTIONSDLG_FONTPANEL_NUMSETTINGS		6	// the number of font-settings rows

// the idoffsets for each font-setting row
#define OPTIONSDLG_FONTPANEL_FONTFACE_IDOFFSET			1
#define OPTIONSDLG_FONTPANEL_FONTSIZE_IDOFFSET			2
#define OPTIONSDLG_FONTPANEL_FGCOL_IDOFFSET				3
#define OPTIONSDLG_FONTPANEL_BKCOL_IDOFFSET				4

// the number of previous idoffsets defined...
#define OPTIONSDLG_FONTPANEL_SETTING_IDOFFSET			5




//! A little class which is used by mgOptFontSettingsPanel to get a list
//! of the fonts available on the current system.
class mgOptFontEnumerator : public wxFontEnumerator
{
	// The array where all font faces are stored.
	wxArrayString m_facenames;

public:
    int GetCount() const
	{ return m_facenames.GetCount(); }
	
    const wxArrayString &GetFacenames() const
	{ return m_facenames; }
	
    wxString *GetFacenamesArray() const { 
		
		// return a wxString[], useful for the controls which
		// take one in their ctor.	You must delete[] it yourself
		// once you are done with it.	Will return NULL if the
		// ArrayString was empty.
		return m_facenames.GetStringArray();		
	}
	
protected:

	// The overridden wxFontEnumerator function
    virtual bool OnFacename(const wxString& facename) {
        m_facenames.Add(facename);
        return TRUE;
    }
};



//! The panel which handles font options inside a mgOptDlg.
//! This panel contains a great number of controls due to the high number
//! of customizable variables regarding the fonts, their foreground and
//! background colors.
//! The panel also contains a preview window which helps the user
//! to choose its preferred settings.
class mgOptFontSettingsPanel : public mgOptBasePanel
{
protected:		// member variables

	//! The IDs usd for the first control placed in each font settings 
	//! row (see #BuildFontSettings for more info).
	//! All the following controls in the same font settings panel uses
	//! their entry in this array as base id, incrementing it
	//! (using the OPTIONSDLG_FONTPANEL_*****_IDOFFSET defines).
	//! All the entries of this array contains 
	//! OPTIONSDLG_FONTPANEL_SETTING_IDOFFSET multiples, so this is the
	//! maximum number of IDs which can be used inside each row.
	int m_nBaseID[OPTIONSDLG_FONTPANEL_NUMSETTINGS];

	//! The labels of the font settings rows.
	//! This array is trictly tied to the mcStyle::GetGetIndexFor() function.
	static wxString m_strSettingsLabel[OPTIONSDLG_FONTPANEL_NUMSETTINGS];

	//! The mgMathBox which is used to show a preview of the fonts to
	//! the user. The input-system and the selection system of this
	//! box is diabled so it works in read-only mode.
	mgMathBox *m_pPreviewBox;
	
	//! The listbox with the exponent depth codes which can be chosen.
	wxChoice *m_pExpDepthChoice;

	//! The last array retrieved by #m_pExpDepthChoice.
	wxString *m_pFontFaces;

	//! The font list available on this system.
	mgOptFontEnumerator m_fntEnumerator;


protected:		// functions

	virtual void Setup( const wxString &itemLabel );
	virtual mgOptDlg::mgOptChangedType CommitChanges( wxConfig* config );

	//! Creates all the controls contained in this panel; specially the
	//! font settings rows.
	void BuildCtrls();

	//! Builds a font settings row, which contains these controls:
	//! | label | wxComboBox with facename list | pointsize | fg colour | bg colour |.
	//! The ID of the first control of the new font settings row is the given one.
	//! All the controls are added inside the given sizer.
	void BuildFontSettings(wxFlexGridSizer *table, const wxString &label, int id);

	//! Updates the controls of the font settings row whose base ID matches the
	//! given ID, reading the informations from the given mcTextSettings object.
	void ReadSettings(int id, const mcTextSettings &p);

	//! Stores inside the given mcTextSettings object, the informations of the 
	//! controls of the font settings row whose  base ID matches the given ID.
	void SaveSettings(int id, mcTextSettings &p);

	//! Handles changes in the exponent depth combo box.
	void OnExpDepthChange(wxCommandEvent &);

	//! Handles the change of the active element of the preview window.
	void OnActiveElemChange(wxCommandEvent &ev);

	//! Returns the mcTextSettings object associated with the font settings row
	//! whose base ID matches the given ID.
	//! \param p The style where the mcTextSettings object is taken from.
	const mcTextSettings &GetSettingsFromBaseID(const mcStyle *p, int id);

	//! Returns the mcTextSettings object associated with the font settings row
	//! whose base ID matches the given ID.
	//! \param expdepth The exponent depth which is used to choose the
	//!        style where the mcTextSettings object is taken from.
	const mcTextSettings &GetSettingsFromBaseID(int expdepth, int id);
	
public:

	mgOptFontSettingsPanel();
	virtual ~mgOptFontSettingsPanel();
	
	
private:
	DECLARE_DYNAMIC_CLASS( mgOptFontSettingsPanel )
	DECLARE_EVENT_TABLE()
};


#endif          // not defined OPTFONTSETTINGS_H

