///////////////////////////////////////////////////////////////////////
// MathGUI = a WYSIWYG equation editor + a powerful math engine      //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
// \file  OptKeyBindings.h
// \brief Defines the mgOptKeyBindingsPanel class.
//
// See OptKeyBindings.cpp for more info.
///////////////////////////////////////////////////////////////////////




#ifndef OPTKEYBINDINGS_H
#define OPTKEYBINDINGS_H


#if defined(__GNUG__) && !defined(__APPLE__)
#pragma interface "OptKeyBindings.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
/*#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

// for all others, include the necessary headers
#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

*/

// includes
#include "mg/OptBasePanel.h"
#include <wx/keybinder.h>	// our base class (a wxWidgets extension)





//! This panel wraps the wxKeyConfigPanel class which is used to
//! allow the user to modify ANY shortcut key used in the menu system
//! of the application which is running.
//! 
//! To allow the presence of the menu-shortcuts in this panel, the
//! application using MathGUI must use wxCustomMenu instead of wxMenu
//! when creating the menu bar:
//!
//! \code
//!     // get a pointer to the global (static) key binder
//! 	wxKeyProfile *main = mgOptKeyBindingsPanel::GetBinder();
//!
//!     // then, all your menus can be created, maybe using also
//!     // the mgMathMenu class...
//!	    wxCustomMenu *file = new wxCustomMenu();
//!     file->Append(id, wxT("menu item name"), "menu help string");
//!     mgMathMenu *p = new mgMathMenu(wnd, box, this, &wxGetApp());
//!     p->AppendFileCommands(file);
//!
//!     // and finally, the menu must be appended to the wxKeyProfile
//!     // (instead of wxMenuBar).
//! 	main->Append(file, "&File");	// VERY IMPORTANT: this call must be
//!                                     // done AFTER all menu items have been
//!                                     // appended to 'file'
//!
//!     // last, we ask our wxKeyProfile to create the menu bar for this frame...
//! 	main->CreateMenuBar(this);
//! 
//! \endcode
class mgOptKeyBindingsPanel : public mgOptBasePanel
{
protected:			// member variables

	//! The key binder handler. Use the #Setup/#SetBinder function to create/set it
	//! and don't forget to use the #Cleanup function to delete it !!!
	static wxKeyProfile *m_pBinder;

	//! The panel which modifies key bindings.
	wxKeyConfigPanel *m_pKeyConfig;

	//! The frame whose menu bar is handled by this panel.
	wxFrame *m_pFrame;

protected:

	virtual mgOptDlg::mgOptChangedType
		CommitChanges( wxConfig* config );
	virtual void Setup( const wxString &itemLabel );

public:
	mgOptKeyBindingsPanel();
	virtual ~mgOptKeyBindingsPanel() {}


	//! Creates the wxKeyProfile.
	//! Use the #Cleanup() function to delete it when it's no longer
	//! necessary; this is an important step to avoid huge memory leaks.
	static void Setup() {
		SetBinder(new wxKeyProfile());
	}

	//! Deletes the wxKeyProfile associated with this panel.
	static void Cleanup() {
		mgSAFE_DELETE(m_pBinder);
	}

	//! Replaces the current global key binder with the given one.
	//! This function can be used to force mgOptKeyBindingsPanel to use
	//! a wxKeyProfile-derived class instead of wxKeyProfile.
	static void SetBinder(wxKeyProfile *p) {
		mgSAFE_DELETE(m_pBinder);
		m_pBinder = p;
	}

	//! Returns the STATIC instance of the wxKeyProfile associated to this
	//! panel. The user of MathGUI should use it to append menu items;
	//! see the class description of mgOptKeyBindingsPanel for more info.
	static wxKeyProfile *GetBinder() {
		wxASSERT_MSG(m_pBinder, wxT("Before using GetBinder(), use Setup()"));
		return m_pBinder;
	}


	static wxMenuBar *m_pMenuBar;

	static void ImportMenuBarCmd(wxMenuBar *p) {
		m_pMenuBar = p;
		GetBinder()->ImportMenuBarCmd(p);
	}


	//! Shows this panel.
	virtual bool Show(bool show = TRUE);

		
private:
	DECLARE_DYNAMIC_CLASS( mgOptKeyBindingsPanel )
	DECLARE_EVENT_TABLE()
};

#endif          // not defined OPTKEYBINDINGS_H


