///////////////////////////////////////////////////////////////////////
// MathGUI = a WYSIWYG equation editor + a powerful math engine      //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   PaletteDlg.h
//! \brief  Defines the mgButtonList class.
//!
//! See PaletteDlg.cpp for more info.
///////////////////////////////////////////////////////////////////////


#ifndef PALETTEDLG_H
#define PALETTEDLG_H

// with this little class, we can create a mgButtonListPanel able
// to send commands both to a single, fixed mgMathBox or the
// mgMathBox which is the active one in a mgMathWnd...
#include "mg/MathBoxWndUser.h"

// this allows us to enable/disable the use of wxPaletteFrame
#ifdef mgUSE_PALETTEFRM
	#include <wx/palettefrm.h>
	#include <wx/minibtn.h>
#else
	//#define wxMainFrame						wxFrame
	#define wxPaletteFrame					wxMiniFrame
	#define wxPALETTEFRM_DEFAULT_STYLE		wxDEFAULT_FRAME_STYLE	
	#include <wx/minifram.h>

	class wxMainFrame : public wxFrame
	{
	public:
		wxMainFrame(wxWindow* parent, wxWindowID id, const wxString& title, 
			const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, 
			long style = wxDEFAULT_FRAME_STYLE, const wxString& name = wxT("frame"))
			: wxFrame(parent, id, title, pos, size, style, name) {}
		
		void InitPalContextMenu() {}
	};

#endif


// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "PaletteDlg.h"
#endif


#define BUTTONLISTPANEL_MAX_HORIZONTAL_LINES			10


//! To use this little class, you should:
//! 1) derive a new class from this one, implementing the
//!    functions grouped under the "Abstract functions" section.
//! 2) in the constructor of the derived class, you should call:
//!    BuildSizer()
//!    [add the buttons to the sizer of the panel]
//!    BuildPanel()
//!

//! A little class which makes the creation of "MathGUI palettes"
//! much easier. For "palettes" I mean those dialog boxes with
//! a smaller title bar, containing various buttons which can
//! be used by the user to insert/edit/manipulate the math data
//! of the main mgMathBox/mgMathWnd object.
//! They are, thus, a sort of toolbars.
//!
//! To keep the same look & feel for all the palettes, all the
//! palettes should derive from this class, which provides the
//! default creation settings.
class mgButtonListPanel : public wxPanel, public mgMathBoxWndUser
{
protected:	// member variables

	//! The list of the wxSizers used to contain the buttons.
	//! Each of them is separed by the previous with a wxStaticLine.
	//wxSizer *m_pSizerList[BUTTONLISTPANEL_MAX_HORIZONTAL_LINES];
	//int m_nUsedSizers;
	int m_nColumns, m_nRows;
	int m_nBaseID;

	//! The window which is used to init the client windows of 
	//! the wxPaletteButtons; that is, the window where focus is
	//! set when the button is clicked.
	wxWindow *m_pClientWnd;

	//! TRUE if all the windows, controls, sizers .... have been
	//! added and the palette is ready to work.
	bool m_bBuildComplete;

	//! The font which is used to draw the button labels.
	wxFont m_fButtons;
	float m_fButtonFontRatio;
	bool m_bEnableFontButtonUpdate;


protected:	// functions

	/////////////////////////////////////////////////////////////////////
	//! \name Utilities
	//! Some functions which manipulate the sizer. @{

	//! Sets all the rows of the main sizer as growable and then
	//! sets up the main sizer of the palette.
	virtual void BuildPanel(int border = 5, int proportion = 30);

	//! Builds the main sizer of this palette. Derived-classes should
	//! call this function as first thing in their constructors.
	virtual void BuildSizer();

	//! Adds to the main sizer a row of spacers.
	void AddHorizontalSpacer();

	//! Adds 'num' buttons to the main sizer, with the given size;
	//! Retrieves the button label and tooltop using the 
	//! #GetButtonLabel and #GetButtonTooltip functions.
	void AddButtonsToSizer(int idoffset, int num, wxSize sz);

	//! Using the font for the button labels, finds the biggest
	//! character of the given string and then returns its size.
	wxSize FindBiggestLetter(const wxString &str);

	//! Returns the default width of this window if each character of
	//! the given string is a toolbutton.
	int GetDefaultWidth(const wxString &str) {
		wxScreenDC dc;
		dc.SetFont(m_fButtons);
		return FindBiggestLetter(str).GetWidth()*GetColCount();
	}

	int GetDefaultHeight(const wxString &str) {
		wxScreenDC dc;
		dc.SetFont(m_fButtons);
		return FindBiggestLetter(str).GetHeight()*GetRowCount();
	}

	void SetFontButtonPointSize(int pointsize);
	void UpdateFontButton();
	void OnSize(wxSizeEvent &);

	//! Resets the client windows of the wxPaletteButtons which are
	//! built through the AddButtonsToSizer...
	void SetClientWindow(wxWindow *p) { m_pClientWnd = p; }

	//! Returns the index of the pressed button with the
	//! given event. The index is zero-based.
	virtual int GetButtonID(wxCommandEvent &) const;

	int GetColCount() const		{ return m_nColumns; }
	int GetRowCount() const		{ return m_nRows; }

	//! Returns the wxWidgets ID of the first button of the 
	//! palette; all the following buttons' ID are obtained
	//! increasing this value.
	int GetBaseID() const		{ return m_nBaseID; }

	//@}



	/////////////////////////////////////////////////////////////////////
	//! \name Abstract functions
	//! The abstract functions which must be implemented in derived classes. @{

	//! Returns the label of the n-th button.
	virtual wxString GetButtonLabel(int n) = 0;

	//! Returns the tooltip for the n-th button.
	virtual wxString GetButtonTooltip(int n) = 0;

	//@}


public:

	//! Constructs this dialog. This function works as the standard
	//! wxWindow constructors, with just two new parameters:
	//! \param mathwnd The math window whose selected mathbox receives
	//!                the commands.
	//! \param mathbox The math box which receives the 
	//!                'create greek symbol' commands.
	mgButtonListPanel(
		wxWindow *parent,
		wxWindow *client,
		mgMathWnd *mathwnd,
		mgMathBox *mathbox,
		int cols, int rows,
		int baseid,
		wxWindowID id = -1, 
		const wxPoint &pos = wxDefaultPosition,
		const wxSize &size = wxDefaultSize,
		long style = wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER) :
			wxPanel(parent, id, pos, size, style), 
			mgMathBoxWndUser(mathwnd, mathbox) {
		
		m_nColumns = cols;
		m_nRows = rows;
		m_nBaseID = baseid;
		m_pClientWnd = client;		
		m_bBuildComplete = FALSE;
		m_bEnableFontButtonUpdate = FALSE;
		m_fButtonFontRatio = 0.0;
		//m_nUsedSizers = 0;
	}

	virtual ~mgButtonListPanel() {
		// the wxWidgets framework will delete the main sizer...
	}


private:
	DECLARE_CLASS(mgButtonListPanel)
	DECLARE_EVENT_TABLE()
};


#endif	// PALETTE_H

